class Photo < ActiveRecord::Base
	serialize :style, PhotoStyle

	belongs_to :user

	if Rails.env == 'production'
  	has_attached_file :image,
      :storage => 's3',
      :s3_credentials => { :access_key_id => ENV['S3_KEY'], :secret_access_key => ENV['S3_SECRET'] },
      :s3_headers => { 'Expires' => 1.year.from_now.httpdate },
      :s3_host_alias => 'photos-p.3-6-5-days.com',
      :bucket => 'photos-p.3-6-5-days.com',
      :url => ":s3_alias_url",
      :path => "/images/:class/:id_:basename.:style.:extension",
      :styles => { :thumb  => '40x40#',
      							:medium => '350x350',
      							:big => '600x600',
      							:original => '1000x1000>' },
      :default_style => :original,
      :convert_options => { :all => '-strip -trim',
                    :thumb => '-quality 85',
                    :medium => '-quality 85',
                    :big => '-quality 88'
                    }
  else
  	has_attached_file :image,
      :styles => { :thumb  => '40x40#',
      							:medium => '350x350',
      							:big => '600x600',
      							:original => '1000x1000>' },
      :default_style => :original,
      :path => ":rails_root/public/system/images/:id/:style/:basename.:extension",
      :convert_options => { :all => '-strip -trim',
                    :thumb => '-quality 85',
                    :medium => '-quality 85',
                    :big => '-quality 88'
                    }
  end

	# paperclip validations

  validates_attachment_presence :image
	validates_attachment_content_type :image, :content_type => ['image/jpeg', 'image/png']

	# other model validations

  validates_length_of :title, :maximum => 30
	validates_with DateUniqueValidator
	validates_presence_of :user_id

  # public methods

  def next(is_admin)
  	if is_admin
  		Photo.order('date ASC').where("date > :current_picture", { :current_picture => self.date }).limit(1).first
  	else
			Photo.order('date ASC').where(["date > ? and date <= ?", self.date, Time.zone.now]).limit(1).first
		end
  end

  def prev
  	Photo.order("date DESC").where(["date < ?", self.date]).limit(1).first
  end

  def in_future?
  	self.date.future?
  end

  def published?
  	!self.date.future?
  end

  # getters and setters

	def style
    # make sure we always return a UserPreference instance
    if read_attribute(:style).nil?
      write_attribute :style, PhotoStyle.new
      read_attribute :style
    else
      read_attribute :style
    end
  end

  def style=(val)
    write_attribute :style, val
  end

  # hooks

	before_create :ensure_style_exists

  # class methods

  def self.page_size
    10
  end

  def self.get_last_published
	  Photo.order("date DESC").where(["date <= ?", Time.zone.now]).limit(1).first
  end

  def self.for_year(date, is_admin, page)
    where_condition = ''
    if is_admin
      where_condition = "date >= :start_date AND date <= :end_date"
      where_data = {:start_date => date.beginning_of_year.beginning_of_day,
		  :end_date => date.end_of_year}
    else
      where_condition = "date >= :start_date AND date <= :end_date and date <= :today"
      where_data = {:start_date => date.beginning_of_year.beginning_of_day,
		  :end_date => date.end_of_year,
		  :today => Time.zone.now }
    end


  	unless page.nil?
			Photo.where(where_condition, where_data).order('date desc').offset(page.to_i * Photo.page_size).limit(Photo.page_size)
		else
      Photo.where(where_condition, where_data).order('date desc')
		end
  end

  def self.for_month( date )
    where_condition = "date >= :start_date AND date <= :end_date"
    where_data = {:start_date => date.beginning_of_month.beginning_of_day,
	  :end_date => date.end_of_month }

	  Photo.where( where_condition, where_data )
  end

  def self.get_last(top)
  	Photo.where("date <= :today", { :today => Time.zone.now }).order('date DESC').limit(top)
  end

  def self.get_next_date
		if Photo.count == 0
			DateTime.now
		else
			Photo.maximum("date") + 1.day
		end
	end

	def height( style=:big )
	  Paperclip::Geometry.from_file(self.image.to_file( style )).height
  end

  def width( style=:big )
	  Paperclip::Geometry.from_file(self.image.to_file( style )).width
  end

  def url( style=:original )
    if Rails.env == 'production'
      # use Amazon cloud front URL instead of S3 URL
      self.image.url( style ).gsub('photos-p', 'photocache')
    else
      self.image.url( style )
    end
  end


  # private methods

  private

  def ensure_style_exists
	  self.style = PhotoStyle.new if self.style.nil?
  end



end

